@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-#
 from django.contrib.auth.models import User
 from django.contrib.contenttypes.models import ContentType
 from django.core.urlresolvers import reverse
@@ -7,6 +8,7 @@ from django.test import TestCase
 from ratings.models import RatedItem
 from ratings.ratings_tests.models import Food, Beverage, BeverageRating
 from ratings.utils import sim_euclidean_distance, sim_pearson_correlation, top_matches, recommendations, calculate_similar_items, recommended_items
+from ratings import views as ratings_views
 
 
 class RatingsTestCase(TestCase):
@@ -24,6 +26,12 @@ class RatingsTestCase(TestCase):
         
         self.related_name = self.rating_model.user.field.related_query_name()
 
+        self._orig_setting = ratings_views.ALLOW_GET
+        ratings_views.ALLOW_GET = False
+
+    def tearDown(self):
+        ratings_views.ALLOW_GET = self._orig_setting
+
     def _sort_by_pk(self, list_or_qs):
         # decorate, sort, undecorate using the pk of the items
         # in the list or queryset
@@ -97,6 +105,10 @@ class RatingsTestCase(TestCase):
         self.assertEqual(self.item1.ratings.count(), 1)
         self.assertEqual(self.item1.ratings.all()[0], rating2)
         
+        # trying to remove multiple times is fine
+        self.item1.ratings.unrate(self.john)
+        self.assertEqual(self.item1.ratings.count(), 1)
+        
         # make sure the item2's rating is still intact
         self.assertEqual(self.item2.ratings.count(), 1)
         
@@ -183,43 +195,109 @@ class RatingsTestCase(TestCase):
         self.assertEqual(rated_qs[1].score, 1.0)
     
     def test_ordering(self):
+        # item 1 has a cumulative score of 0
         rating1 = self.item1.ratings.rate(self.john, 1)
         rating2 = self.item1.ratings.rate(self.jane, -1)
+        
+        # item 2 has a score of 1
         rating3 = self.item2.ratings.rate(self.john, 1)
+
+        # item 3 has no ratings
+        self.item3 = self.rated_model.objects.create(name='item3')
         
+        # get a queryset of all items ordered by rating
         rated_qs = self.rated_model.ratings.all().order_by_rating()
-        self.assertQuerySetEqual(rated_qs, [self.item2, self.item1])
-        self.assertQuerySetEqual(rated_qs,
-            self.rated_model.ratings.order_by_rating()
-        )
         
+        # check that it is ordered as we expect, descending with nulls last
+        self.assertEqual(list(rated_qs), [self.item2, self.item1, self.item3])
+        
+        # check that it is equivalent to the model method
+        self.assertEqual(list(rated_qs), list(
+            self.rated_model.ratings.order_by_rating()
+        ))
+        
+        # check that passing in a queryset of all objects results in the same
+        # ordering as when it is queried without an inner queryset
+        alt_rated_qs = self.rated_model.ratings.all().order_by_rating(
+            queryset=self.rated_model.objects.all()
+        )
+        self.assertEqual(list(alt_rated_qs), list(rated_qs))
+        
+        # check that the scores are what we expect them to be
         self.assertEqual(rated_qs[0].score, 1)
         self.assertEqual(rated_qs[1].score, 0)
+        self.assertEqual(rated_qs[2].score, None)
         
-        item1_qs = self.rated_model._default_manager.filter(pk=self.item1.pk)
-        rated_qs = self.rated_model.ratings.all().order_by_rating(queryset=item1_qs)
-        self.assertQuerySetEqual(rated_qs, [self.item1])
+        # restrict the queryset to only contain item 1 and item 3
+        item13_qs = self.rated_model._default_manager.filter(pk__in=[
+            self.item1.pk, self.item3.pk
+        ])
+        
+        # get model ordered by rating restricted to our queryset
+        rated_qs = self.rated_model.ratings.all().order_by_rating(queryset=item13_qs)
+        
+        # should contain just the two items we're interested in
+        self.assertEqual(list(rated_qs), [self.item1, self.item3])
+        
+        # check that the model method results are what we expect
         self.assertQuerySetEqual(rated_qs,
-            self.rated_model.ratings.order_by_rating(queryset=item1_qs)
+            self.rated_model.ratings.order_by_rating(queryset=item13_qs)
         )
-        
+
+        # check that the scores are correct
         self.assertEqual(rated_qs[0].score, 0)
+        self.assertEqual(rated_qs[1].score, None)
         
+        # try ordering by score ascending -- should now be nulls first.  also
+        # use an alias for the aggregator
         rated_qs = self.rated_model.ratings.all().order_by_rating(descending=False, alias='sum_score')
-        self.assertQuerySetEqual(rated_qs, [self.item1, self.item2])
-        self.assertQuerySetEqual(rated_qs,
+        
+        # check that they're ordered correctly
+        self.assertEqual(list(rated_qs), [self.item3, self.item1, self.item2])
+        
+        # conforms to the other api
+        self.assertEqual(list(rated_qs), list(
             self.rated_model.ratings.order_by_rating(descending=False, alias='sum_score')
-        )
+        ))
         
-        self.assertEqual(rated_qs[0].sum_score, 0)
-        self.assertEqual(rated_qs[1].sum_score, 1)
+        # extra attributes are set correctly
+        self.assertEqual(rated_qs[0].sum_score, None)
+        self.assertEqual(rated_qs[1].sum_score, 0)
+        self.assertEqual(rated_qs[2].sum_score, 1)
         
+        # changing a rating results in different ordering (ths is just a sanity check)
         self.item1.ratings.rate(self.john, 3)
         rated_qs = self.rated_model.ratings.all().order_by_rating()
-        self.assertQuerySetEqual(rated_qs, [self.item1, self.item2])
+        self.assertEqual(list(rated_qs), [self.item1, self.item2, self.item3])
     
         self.assertEqual(rated_qs[0].score, 2)
         self.assertEqual(rated_qs[1].score, 1)
+
+    def test_ordering_with_filter(self):
+        # item 1 has a cumulative score of 0
+        rating1 = self.item1.ratings.rate(self.john, 1)
+        rating2 = self.item1.ratings.rate(self.jane, -1)
+        
+        # item 2 has a score of 1
+        rating3 = self.item2.ratings.rate(self.john, 2)
+
+        # see what john has rated
+        rated_qs = self.rated_model.ratings.filter(user=self.john).order_by_rating()
+
+        self.assertEqual(list(rated_qs), [self.item2, self.item1])
+        r3, r1 = rated_qs
+        self.assertEqual(r3.score, 2.0)
+        self.assertEqual(r1.score, 1.0)
+
+        # change the rating and see it reflected in new query
+        self.item1.ratings.rate(self.john, 4)
+
+        rated_qs = self.rated_model.ratings.filter(user=self.john).order_by_rating()
+
+        self.assertEqual(list(rated_qs), [self.item1, self.item2])
+        r1, r3 = rated_qs
+        self.assertEqual(r1.score, 4.0)
+        self.assertEqual(r3.score, 2.0)
     
     def test_rating_score_filter(self):
         t = Template('{% load ratings_tags %}{{ obj|rating_score:user }}')
@@ -296,7 +374,7 @@ class RatingsTestCase(TestCase):
         
         # log in
         self.client.login(username='a', password='a')
-        
+
         # hit the view with a GET
         resp = self.client.get(test_url)
         self.assertEqual(resp.status_code, 405) # bad method, yo
@@ -345,6 +423,36 @@ class RatingsTestCase(TestCase):
         self.assertEqual(resp.status_code, 200)
         
         self.assertEqual(self.item1.ratings.cumulative_score(), 2.5)
+        
+        # hit it with a negative rating
+        test_url = reverse('ratings_rate_object', args=(
+            ctype.pk,
+            self.item1.pk,
+            -1.5,
+        ))
+        
+        resp = self.client.post(test_url, {}, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
+        self.assertEqual(resp.status_code, 200)
+        
+        self.assertEqual(self.item1.ratings.cumulative_score(), 1.0)
+
+        # finally, test that we can hit it with a GET
+        ratings_views.ALLOW_GET = True
+
+        test_url = reverse('ratings_rate_object', args=(
+            ctype.pk,
+            self.item1.pk,
+            100.0,
+        ))
+        resp = self.client.get(test_url, {'next': '/'})
+        self.assertEqual(resp.status_code, 302)
+
+        self.assertEqual(self.item1.ratings.cumulative_score(), 102.5)
+
+    def test_rated_item_model_unicode(self):
+        self.john.username = u'Иван'
+        rating = self.item1.ratings.rate(self.john, 1)
+        rating_unicode_string = unicode(rating)
 
 
 class CustomModelRatingsTestCase(RatingsTestCase):
@@ -462,3 +570,10 @@ class RecommendationsTestCase(TestCase):
         
         self.assertEqual(str(r2[0])[:5], '2.084')
         self.assertEqual(r2[1], self.food_e)
+
+    def test_similar_item_model_unicode(self):
+        self.food_b.name = u'яблоко'
+        self.food_b.save()
+        calculate_similar_items(RatedItem.objects.all(), 10)
+        top_for_food_a = self.food_a.ratings.similar_items()[0]
+        top_for_food_a_unicode_string = unicode(top_for_food_a)
